package id.co.sangkuriang.inovasi.daerah.master.utility

import com.fasterxml.jackson.databind.ObjectMapper
import id.co.sangkuriang.inovasi.daerah.master.constant.*
import id.co.sangkuriang.inovasi.daerah.master.pojo.Users
import org.apache.commons.io.FileUtils
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.multipart.MultipartFile
import java.io.File
import java.io.IOException
import java.net.MalformedURLException
import java.net.URL
import java.nio.file.Files
import java.nio.file.Path
import java.nio.file.Paths
import javax.servlet.http.HttpServletRequest

fun getUsersLogin(): Users? {
    return try {
        val principal = SecurityContextHolder.getContext().authentication.principal as Any
        ObjectMapper().readValue(principal.toString(), Users::class.java)
    } catch (e: Exception) {
        null
    }
}

@Throws(MalformedURLException::class)
fun getBaseUrl(request: HttpServletRequest): String {
    try {
        val requestURL = URL(request.requestURL.toString())
        val port = if (requestURL.port == -1) "" else ":" + requestURL.port
        return requestURL.protocol + "://" + requestURL.host + port + request.contextPath + "/"
    } catch (e: Exception) {
        throw e
    }
}

fun getFullUrl(request: HttpServletRequest): String {
    val requestURL = StringBuilder(request.requestURL.toString())
    val queryString = request.queryString
    return if (queryString == null) {
        requestURL.toString()
    } else {
        requestURL.append('?').append(queryString).toString()
    }
}

fun getPathUrl(request: HttpServletRequest): String {
    return getFullUrl(request).replace(getBaseUrl(request), "")
        .replaceAfter("?", "")
        .replace("?", "")
        .replaceAfter("/", "")
        .replace("/", "")
}


fun uploadFile(
    file: MultipartFile,
    httpServletRequest: HttpServletRequest,
    path: String,
    fileType: String? = "images"
): String {
    try {
        var dirType = ""
        when (path) {
            UPLOAD_USER_DIR -> {
                dirType = "users"
            }
            UPLOAD_APPLICATION_DIR -> {
                dirType = "application"
            }
            UPLOAD_ICON_APPLICATION_CATEGORY_DIR -> {
                dirType = "application-category-icon"
            }
            UPLOAD_IMAGE_APPLICATION_CATEGORY_DIR -> {
                dirType = "application-category-image"
            }
            UPLOAD_BANNER_APPLICATION_DIR -> {
                dirType = "application-banner"
            }
            UPLOAD_IMAGE_APPLICATION_QR_DIR -> {
                dirType = "application-qr"
            }
            UPLOAD_IMAGE_APPLICATION_DETAIL_DIR -> {
                dirType = "application-detail"
            }
        }

        val root: Path = Paths.get(path)
        if (!Files.exists(root)) {
            try {
                Files.createDirectories(Paths.get(path))
            } catch (e: IOException) {
                throw RuntimeException("Could not create upload folder!")
            }
        }

        val fileName =
            System.currentTimeMillis().toString() + "-" + file.originalFilename?.trim()?.replace(" ", "-")
        Files.copy(file.inputStream, root.resolve(fileName))

        return getBaseUrl(httpServletRequest) + "file/$fileType?type=$dirType&fileName=" + fileName
    } catch (e: Exception) {
        throw RuntimeException("Could not store the file. Error: " + e.message)
    }
}

fun deleteFile(fileDir: String, file: String) {
    try {
        val delete = File(fileDir + File.separator + file.substringAfterLast("fileName="))
        if (delete.exists()) {
            FileUtils.forceDelete(delete)
        }
    } catch (e: Exception) {
        throw e
    }
}
